﻿local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("EnsidiaFails")
local BB = LibStub("LibBabble-Boss-3.0"):GetUnstrictLookupTable()
local BZ = LibStub("LibBabble-Zone-3.0"):GetUnstrictLookupTable()
EnsidiaFails = CreateFrame("Frame")
local addon = EnsidiaFails
LibStub("AceConsole-3.0"):Embed(addon)
LibStub("LibSimpleTimer-1.0"):Embed(addon)

local announce_to = {
	SAY = L["Say"],
	RAID = L["Raid"],
	GUILD = L["Guild"],
	PARTY = L["Party"],
	OFFICER = L["Officer"],
	CHANNEL = L["Channel"],
	SELF = L["Self"],
}

local announce_style = {
	during = L["during"],
	after = L["after"],
	during_and_after = L["during_and_after"],
}

function addon:Initialize()
	if FailCount == nil then
		FailCount = {}
	elseif OFailCount == nil then
		OFailCount = {}
	end
end	

function addon:MenuButtonRemove(info, input)
	if self.db.profile.announce == "CHANNEL" then
		whisper=self.db.profile.channelnumber
		target=self.db.profile.announce
	else
		target=self.db.profile.announce
	end
	local firstletter = input:sub(0,1):upper()
	local rest = input:sub(2)
	local p = firstletter..rest		
	addon:RemoveFail(p,target,whisper)
end

function addon:OpenMenu(input)
	if input == "" then InterfaceOptionsFrame_OpenToCategory("EnsidiaFails")
	elseif input == "wipe" then
		addon:Wipe()
	elseif input=="reset" then
		addon:reset()
	elseif input=="oreset" then
		addon:oreset()
	elseif input:sub(0,6) == "remove" then
		if self.db.profile.announce == "CHANNEL" then
			whisper=self.db.profile.channelnumber
			target=self.db.profile.announce
		else
			target=self.db.profile.announce
		end
		if input:len() > 6 then
			local name = input:sub(8)
			local firstletter = name:sub(0,1):upper()
			local rest = name:sub(2)
			local p = firstletter..rest		
			addon:RemoveFail(p,target,whisper)
		end
	elseif input=="stats" then
		addon:Stats(addon.db.profile.announce,"stats",addon.db.profile.channelnumber)
	elseif input=="ostats" then
		addon:Stats(addon.db.profile.announce,"ostats",addon.db.profile.channelnumber)
	end
end

function addon:SetChannelNumber(info, input)
	self.db.profile.channelnumber = GetChannelName(input)
end

function addon:GetChannelNumber(input)
	return addon.db.profile.channelnumber
end

addon:SetScript("OnEvent", function(self, event, ...)
	if event == "ADDON_LOADED" then
		local name = ...
		if name:lower() ~= "ensidiafails" then return end
		addon:Initialize()		
		self.db = LibStub("AceDB-3.0"):New("EnsidiaFailsDB", {
			profile = {
				noexceptions = false,
				stats_value = 10,
				stats_all = false,
				devouring_flame = true,
				flame_jets = true,
				void_zone = true,
				lightbomb = true,
				lightbomb_value = 4,
				gravity_bomb = true,
				overload = true,
				overload_value = 5000,
				rune_of_death = true,				
				rune_of_death_value = 2,
				eyebeam = true,
				seeping_feral_essence = true,
				sonic_screech = true,
				ice_shards = true,
				flash_freez = true,
				cold = true,
				cold_value = 2,
				lightning_charge = true,
				chainlightning = true,
				runic_smash = true,
				blizzard = true,
				hardened_bark = true,
				hardened_bark_value = 50,
				natures_fury = true,
				natures_fury_combatlog_time = 0.3,
				natures_fury_move_time = 3,
				nature_bomb = true,
				unstable_energy = true,
				bind_life = true,
				ground_tremor = true,
				flames = true,
				water_spray = true,
				deafening_siren = true,
				mines = true,
				rocket_strike = true,
				laser_barrage = true,
				shock_blast = true,		
				frost_bomb = true,
				bombbots = true,
				napalm_shell = true,
				shadow_crash = true,
				saronite_vapor = true,
				general_healing_himself = true,
				general_healing_himself_value = 350000,
				death_ray = true,
				lunatic_gaze = true,
				malady_of_the_mind = true,
				focused_anger = true,
				focused_anger_value = 50,
				insane = true,
				cosmic_smash = true,
				cosmic_smash_value = 7000,
				big_bang = true,
				black_hole = true,
				lightning_nova = true,
				lightning_nova_value = 20000,
				mutating_injection = true,
				flame_tsunami = true,
				sartharion_void = true,
				eruption = true,
				frogger = true,
				mark = true,
				frost_breath = true,
				polarity_shift = true,				
				jumping = true,		
				hm_void = true,
				kt_void	= true,
				disabled = false,
				announce = "RAID",
				announce_style = "during_and_after",
			},
		}, "Default")
		
		local args = {
			type = "group",
			handler = self,
			get = function(info) return self.db.profile[info[1]] end,
			set = function(info, v) self.db.profile[info[1]] = v end,
			args = {			
				desc = {
					type = "description",
					name = L["addon_desc"],
					order = 1,
					fontSize = "medium",
				},
				reset_header = {
					type = "header",
					name = L["reset"],
					order = 70,
				},
				reset = {
					order = 90,
					type = "execute",
					name = L["reset"],
					desc = L["reset_desc"],
					func = function()
						addon:reset()
					end
				},
				oreset = {
					order = 90,
					type = "execute",
					name = L["oreset"],
					desc = L["oreset_desc"],
					func = function()
						addon:oreset()
					end
				},
				announce_header = {
					type = "header",
					name = L["announce"],
					order = 100,
				},
				announce_style = {
					order = 210,
					type = "select",
					name = L["announce_style"],
					desc = L["announce_style_desc"],
					values = announce_style,
				},
				announce = {
					order = 210,
					type = "select",
					name = L["announce"],
					desc = L["announce_desc"],
					values = announce_to,
				},
				channel = {
					order = 210,
					type = "input",
					name = L["Channel"],
					desc = L["Channel"],
					set = "SetChannelNumber",
					get = "GetChannelNumber",
				},
				disabled = {
					type = "toggle",
					name = L["Disabled"],
					order = 211,
				},
				newline = {
					type = "description",
					name = "\n",
					order = 213,				
				},
				stats = {
					order = 214,
					type = "execute",
					name = L["stats"],
					desc = L["stats_desc"],
					func = function()
						addon:Stats(addon.db.profile.announce,"stats",addon.db.profile.channelnumber)
					end
				},
				ostats = {
					order = 218,
					type = "execute",
					name = L["ostats"],
					desc = L["ostats_desc"],
					func = function()
						addon:Stats(addon.db.profile.announce,"ostats",addon.db.profile.channelnumber)
					end
				},
				stats_value = {
					type = "range",
					order = 219,
					name = L["Top X Stats"],
					desc = L["Amount of entries to display"],
					min = 1, max = 25, step = 1,
					disabled = function() return addon.db.profile.stats_all end,
				},				
				stats_all = {
					type = "toggle",
					name = L["Show all"],
					order = 220	,			
				},
				remove_header = {
					type = "header",
					name = L["remove"],
					order = 225,
				},
				menubuttonremove = {
					order = 330,
					type = "input",
					name = L["remove"],
					desc = L["remove_desc"],
					set = "MenuButtonRemove",
				},
			},
		}
			
		LibStub("AceConfig-3.0"):RegisterOptionsTable("EnsidiaFails", args)
		LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("EnsidiaFails-Filter", self.GenerateFilterOptions)
		LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EnsidiaFails", "EnsidiaFails")
		LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EnsidiaFails-Filter", L["filter"], "EnsidiaFails")
		self:RegisterChatCommand("ef", "OpenMenu")
		self:RegisterChatCommand("EnsidiaFails", "OpenMenu")
	elseif (event=="COMBAT_LOG_EVENT_UNFILTERED") then
		addon:CombatLog(event,...)
	elseif (event=="UNIT_AURA") then
		addon:OnUNIT_AURA(event,...)
	elseif (event=="PLAYER_REGEN_ENABLED") then	
		addon:KeepCheckingForWipe()
		if ((addon.db.profile.announce_style == "after") or (addon.db.profile.announce_style == "during_and_after")) then			
			addon:AnnounceAfter()
		end
	else--if event == "CHAT_MSG_RAID_LEADER" or "CHAT_MSG_RAID" or "CHAT_MSG_PARTY" or "CHAT_MSG_SAY" or "CHAT_MSG_GUILD" or "CHAT_MSG_CHANNEL" then
		addon:ChatEventHandler(event, ...)
	end
end)

function addon.GenerateFilterOptions()
	if not addon.FilterOptions then
		addon.GenerateFilterOptionsInternal()
		addon.GenerateFilterOptionsInternal = nil
		moduleFilterOptions = nil
	end
	return addon.FilterOptions
end

function addon.GenerateFilterOptionsInternal()
	addon.FilterOptions = {
		type = "group",
		name = "Filter",
		get = function(info) return addon.db.profile[ info[#info] ] end,
		set = function(info, value) addon.db.profile[ info[#info] ] = value end,
		args = {
			General = {
				order = 1,
				type = "group",
				name = L["general"],
				args = {
					desc = {
						type = "description",
						name = L["filter_desc"],
						order = 1,
						fontSize = "large",
						width = "full",
					},
					noexceptions = {
						type = "toggle",
						name = L["noexceptions"],
						desc = L["noexceptions_desc"],
						order = 10,
					},
				},
			},
			Ulduar = {
				order = 2,
				type = "group",
				name = BZ["Ulduar"],
				args = {
					Razorscale = {
						order = 2,
						type = "group",
						name = BB["Razorscale"],
						args = {
							devouring_flame = {
								type = "toggle",
								name = GetSpellInfo(64704),
								order = 20,
							},
						},
					},	
					Ignis = {
						order = 3,
						type = "group",
						name = BB["Ignis the Furnace Master"],
						args = {
							eyebeam = {
								type = "toggle",
								name = GetSpellInfo(62681),
								order = 20,
							},
						},
					},	
					XT = {
						order = 4,
						type = "group",
						name = BB["XT-002 Deconstructor"],
						args = {
							void_zone = {
								type = "toggle",
								name = GetSpellInfo(64208),
								order = 20,
							},
							lightbomb = {
								type = "toggle",
								name = GetSpellInfo(65120),
								order = 30,
							},
							lightbomb_value = {
								type = "range",
								order = 35,
								name = GetSpellInfo(65120),
								desc = L["How much time do you have for moving before adding a fail for "]..(GetSpellInfo(65120)),
								min = 1, max = 10, step = 1,
								disabled = function() return not addon.db.profile.lightbomb end,
							},							
							gravity_bomb = {
								type = "toggle",
								name = GetSpellInfo(64233),
								order = 25,
							},
						},
					},
					Council = {
						order = 5,
						type = "group",
						name = BB["The Iron Council"],
						args = {
							overload = {
								type = "toggle",
								name = GetSpellInfo(61878),
								order = 20,
							},
							overload_value = {
								type = "range",
								order = 25,
								name = GetSpellInfo(61878),
								desc = L["How much damage taken needed for a fail from "]..(GetSpellInfo(61878)),
								min = 4000, max = 10000, step = 500,
								disabled = function() return not addon.db.profile.overload end,
							},							
							rune_of_death = {
								type = "toggle",
								name = GetSpellInfo(63490),
								order = 30,
							},
							rune_of_death_value = {
								type = "range",
								order = 35,
								name = GetSpellInfo(63490),
								desc = L["How much time do you have for moving before adding a fail for "]..(GetSpellInfo(63490)),
								min = 1, max = 5, step = 1,
								disabled = function() return not addon.db.profile.rune_of_death end,
							},								
						},
					},					
					Kologarn = {
						order = 6,
						type = "group",
						name = BB["Kologarn"],
						args = {
							eyebeam = {
								type = "toggle",
								name = GetSpellInfo(63976),
								order = 20,
							},
						},
					},
					Auriaya = {
						order = 7,
						type = "group",
						name = BB["Auriaya"],
						args = {
							seeping_feral_essence = {
								type = "toggle",
								name = GetSpellInfo(64459),
								order = 20,
							},
							sonic_screech = {
								type = "toggle",
								name = GetSpellInfo(64688),
								order = 30,
							},						
						},
					},	
					Hodir = {
						order = 8,
						type = "group",
						name = BB["Hodir"],
						args = {
							ice_shards = {
								type = "toggle",
								name = GetSpellInfo(62457),
								order = 20,
							},
							flash_freez = {
								type = "toggle",
								name = GetSpellInfo(61969),
								order = 30,
							},	
							cold = {
								type = "toggle",
								name = GetSpellInfo(62039),
								order = 40,
							},	
							cold_value = {
								type = "range",
								order = 45,
								name = GetSpellInfo(62039),
								desc = L["How many stack is still not a fail"],
								min = 2, max = 10, step = 1,
								disabled = function() return not addon.db.profile.cold end,
							},								
						},
					},	
					Thorim = {
						order = 9,
						type = "group",
						name = BB["Thorim"],
						args = {
							lightning_charge = {
								type = "toggle",
								name = GetSpellInfo(62466),
								order = 20,
							},
							chainlightning = {
								type = "toggle",
								name = GetSpellInfo(64390),
								order = 30,
							},	
							runic_smash = {
								type = "toggle",
								name = GetSpellInfo(62465),
								order = 40,
							},	
							blizzard = {
								type = "toggle",
								name = GetSpellInfo(62576),
								order = 50,
							},								
						},
					},	
					Freya = {
						order = 10,
						type = "group",
						name = BB["Freya"],
						args = {
							nature_bomb = {
								type = "toggle",
								name = GetSpellInfo(64650),
								order = 20,
							},
							unstable_energy = {
								type = "toggle",
								name = GetSpellInfo(62865),
								order = 30,
							},	
							bind_life = {
								type = "toggle",
								name = GetSpellInfo(63559),
								order = 40,
							},	
							ground_tremor = {
								type = "toggle",
								name = GetSpellInfo(62859),
								order = 50,
							},
							hardened_bark = {
								type = "toggle",
								name = GetSpellInfo(64190),
								order = 60,
							},	
							hardened_bark_value = {
								type = "range",
								order = 65,
								name = GetSpellInfo(64190),
								desc = L["At how many stacks are you supposed to stop dps"],
								min = 1, max = 99, step = 1,
								disabled = function() return not addon.db.profile.hardened_bark end,
							},							
							natures_fury = {
								type = "toggle",
								name = GetSpellInfo(63570),
								order = 70,
							},	
							natures_fury_combatlog_time = {
								type = "range",
								order = 72,
								name = GetSpellInfo(63570),
								desc = L["How close combatlog events have to be, when determining who failed"],
								min = 0.1, max = 0.5, step = 0.05,
								disabled = function() return not addon.db.profile.natures_fury end,
							},
							natures_fury_move_time = {
								type = "range",
								order = 75,
								name = GetSpellInfo(63570),
								desc = L["How much time do you have for moving before adding a fail for "]..(GetSpellInfo(63570)),
								min = 1, max = 5, step = 1,
								disabled = function() return not addon.db.profile.natures_fury end,
							},							
						},
					},
					Mimiron = {
						order = 11,
						type = "group",
						name = BB["Mimiron"],
						args = {
							water_spray = {
								type = "toggle",
								name = GetSpellInfo(64619),
								order = 10,
							},
							deafening_siren = {
								type = "toggle",
								name = GetSpellInfo(64616),
								order = 15,
							},						
							flames = {
								type = "toggle",
								name = GetSpellInfo(64566),
								order = 20,
							},
							mines = {
								type = "toggle",
								name = GetSpellInfo(63027),
								order = 30,
							},	
							rocket_strike = {
								type = "toggle",
								name = GetSpellInfo(63041),
								order = 40,
							},	
							laser_barrage = {
								type = "toggle",
								name = GetSpellInfo(63293),
								order = 50,
							},	
							shock_blast = {
								type = "toggle",
								name = GetSpellInfo(63631),
								order = 60,
							},
							frost_bomb = {
								type = "toggle",
								name = GetSpellInfo(64623),
								order = 70,
							},	
							bombbots = {
								type = "toggle",
								name = GetSpellInfo(63801),
								order = 80,
							},	
							napalm_shell = {
								type = "toggle",
								name = GetSpellInfo(65026),
								order = 90,
							},								
						},
					},	
					General = {
						order = 12,
						type = "group",
						name = BB["General Vezax"],
						args = {
							shadow_crash = {
								type = "toggle",
								name = GetSpellInfo(63277),
								order = 20,
							},
							saronite_vapor = {
								type = "toggle",
								name = GetSpellInfo(63337),
								order = 30,
							},
							general_healing_himself = {
								type = "toggle",
								name = GetSpellInfo(63276),
								order = 40,
							},
							general_healing_himself_value = {
								type = "range",
								order = 45,
								name = GetSpellInfo(63276),
								desc = L["How much healing taken is still not a fail"],
								min = 350000, max = 1000000, step = 50000,
								disabled = function() return not addon.db.profile.general_healing_himself end,
							},
						},
					},	
					Yogg = {
						order = 13,
						type = "group",
						name = BB["Yogg-Saron"],
						args = {
							death_ray = {
								type = "toggle",
								name = GetSpellInfo(63884),
								order = 20,
							},
							lunatic_gaze = {
								type = "toggle",
								name = GetSpellInfo(64164),
								order = 30,
							},
							malady_of_the_mind = {
								type = "toggle",
								name = GetSpellInfo(63881),
								order = 40,
							},
							insane = {
								type = "toggle",
								name = GetSpellInfo(63120),
								order = 50,
							},	
							focused_anger = {
								type = "toggle",
								name = GetSpellInfo(57689),
								order = 60,
							},	
							focused_anger_value = {
								type = "range",
								order = 65,
								name = GetSpellInfo(57689),
								desc = L["At how many stacks are you supposed to stop dps"],
								min = 1, max = 99, step = 1,
								disabled = function() return not addon.db.profile.focused_anger end,
							},								
						},
					},					
					Algalon = {
						order = 14,
						type = "group",
						name = BB["Algalon the Observer"],
						args = {
							cosmic_smash = {
								type = "toggle",
								name = GetSpellInfo(62311),
								order = 40,
							},
							cosmic_smash_value = {
								type = "range",
								order = 45,
								name = GetSpellInfo(62311),
								desc = L["How much damage taken needed for a fail from "]..(GetSpellInfo(62311)),
								min = 5000, max = 15000, step = 1000,
								disabled = function() return not addon.db.profile.cosmic_smash end,
							},								
							black_hole = {
								type = "toggle",
								name = GetSpellInfo(62169),
								order = 30,
							},
							big_bang = {
								type = "toggle",
								name = GetSpellInfo(64584),
								order = 20,
							},
						},
					},					
				},
			},
			Naxxramas = {
				order = 3,
				type = "group",
				name = BZ["Naxxramas"],
				args = {
					Patchwerk = {
						order = 1,
						type = "group",
						name = BB["Patchwerk"],
						args = {
							frogger = {
								type = "toggle",
								name = GetSpellInfo(28433),
								order = 20,
							},
						},
					},				
					Grobbulus = {
						order = 2,
						type = "group",
						name = BB["Grobbulus"],
						args = {
							mutating_injection = {
								type = "toggle",
								name = GetSpellInfo(28169),
								order = 20,
							},
						},
					},
					Heigan = {
						order = 3,
						type = "group",
						name = BB["Heigan the Unclean"],
						args = {
							eruption = {
								type = "toggle",
								name = GetSpellInfo(29371),
								order = 20,
							},
						},
					},
					Thaddius = {
						order = 4,
						type = "group",
						name = BB["Thaddius"],
						args = {
							polarity_shift = {
								type = "toggle",
								name = GetSpellInfo(28089),
								order = 20,
							},
							jumping = {
								type = "toggle",
								name = GetSpellInfo(47018),
								order = 30,
							},
						},
					},
					HorseMan = {
						order = 5,
						type = "group",
						name = BB["The Four Horsemen"],
						args = {
							hm_void = {
								type = "toggle",
								name = GetSpellInfo(28865),
								order = 20,
							},
							mark = {
								type = "toggle",
								name = GetSpellInfo(28836),
								order = 30,
							},
						},
					},
					Sapphiron = {
						order = 6,
						type = "group",
						name = BB["Sapphiron"],
						args = {
							frost_breath = {
								type = "toggle",
								name = GetSpellInfo(28524),
								order = 20,
							},
						},
					},	
					KelThuzad = {
						order = 7,
						type = "group",
						name = BB["Kel'Thuzad"],
						args = {
							frost_breath = {
								type = "toggle",
								name = GetSpellInfo(27812),
								order = 20,
							},
						},
					},					
				},
			},
			Vault = {
				order = 4,
				type = "group",
				name = BZ["Vault of Archavon"],
				args = {
					Hodir = {
						order = 1,
						type = "group",
						name = BB["Emalon the Storm Watcher"],
						args = {
							lightning_nova = {
								type = "toggle",
								name = GetSpellInfo(65279),
								order = 20,
							},
							lightning_nova_value = {
								type = "range",
								order = 25,
								name = GetSpellInfo(65279),
								desc = L["How much damage taken needed for a fail from "]..(GetSpellInfo(65279)),
								min = 15000, max = 30000, step = 1000,
								disabled = function() return not addon.db.profile.lightning_nova end,
							},							
						},
					},
				},
			},
			Sartharion = {
				order = 5,
				type = "group",
				name = BZ["The Obsidian Sanctum"],
				args = {
					Sartharion = {
						order = 1,
						type = "group",
						name = BB["Sartharion"],
						args = {
							flame_tsunami = {
								type = "toggle",
								name = GetSpellInfo(57491),
								order = 20,
							},
							sartharion_void = {
								type = "toggle",
								name = GetSpellInfo(59128),
								order = 30,
							},							
						},
					},
				},
			},			
		}
	}
end

function addon:ChatEventHandler(event, ...)		
	local msg,_,_,_,_,_,_,channelnum = select(1,...)
	local _,channelname = GetChannelName(channelnum)
	msg = msg:lower()
	if event == "CHAT_MSG_RAID_LEADER" then
		if msg:find("!wipe") then
			addon:Wipe()
		end
	elseif event == "CHAT_MSG_RAID" then
		if msg:find("!wipe") then
			addon:Wipe()
		end
	end
	if channelname then
		if channelname:find(L["general"]) or channelname:find(L["Trade"]) or channelname:find(L["LocalDefense"]) then return end
	end
	if msg:find("!stats") then
		target = event:sub(10)
		if target == "RAID_LEADER" then
			target = "RAID"
		end
		local Time = GetTime()
		if LastTime == nil then
			LastTime = Time
			addon:Stats(target,"stats",channelnum)
		elseif LastTime+10.000 < Time then
			addon:Stats(target,"stats",channelnum)
			LastTime=nil
		end				
	elseif msg:find("!ostats") then
		target = event:sub(10)
		if target == "RAID_LEADER" then
			target = "RAID"
		end
		local Time = GetTime()
		if LastTime == nil then
			LastTime = Time
			addon:Stats(target,"ostats",channelnum)
		elseif LastTime+10.000 < Time then
			addon:Stats(target,"ostats",channelnum)
			LastTime=nil
		end	
	end
end


addon:RegisterEvent("PLAYER_REGEN_ENABLED")
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("CHAT_MSG_RAID")
addon:RegisterEvent("CHAT_MSG_RAID_LEADER")
addon:RegisterEvent("CHAT_MSG_PARTY")
addon:RegisterEvent("CHAT_MSG_SAY")
addon:RegisterEvent("CHAT_MSG_GUILD")
addon:RegisterEvent("CHAT_MSG_CHANNEL")
addon:RegisterEvent("UNIT_HEALTH")
addon:RegisterEvent("UNIT_AURA")
addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLog")


-- local variables --
local searing_applied = nil
local hardened_bark = false
local focused_anger = false
local event_fails ={} 
local sorttable ={} 
local failed_at={}
local lastCold = nil
local lastColdTime = nil
local cold = select(1,GetSpellInfo(62039))
local not_failed_on_sonic_counter = 1
local not_failed_on_sonic = {}
local FailsForTheSession = {}
local LunaticGaze = {}
local bigbang_is_casting = false
local three_people_hugging = false
local FailedatGravityBomb = false
local LastTime = nil
local heal_amount = 0
local fury_on = nil
local fury_on1 = nil
local fury_on2 = nil
local LastEvent1 = {}
local LastEvent3 = nil
local LastEvent2 = {}
local ChargeCounter = {}
local shock_blast_casting = false
local ThaddiusAlive = true;
local WightLastAction = 0

function addon:Stats(target, stat, whisper)
	if stat == "ostats" then
		Count = OFailCount
	else
		Count = FailCount
	end
	local count,players=0,0
	local sorttable={}
	for k,v in pairs(Count) do
		count = count + v
		players = players + 1
		table.insert(sorttable, {k,v})
	end
	if (players == 0) then
		if addon.db.profile.announce == "SELF" then
			DEFAULT_CHAT_FRAME:AddMessage(L["nobody_failed"])
		else
			SendChatMessage(L["nobody_failed"],target, nil, whisper)
		end
	else
		table.sort(sorttable,function(a,b) return a[2] > b[2] end)
		if addon.db.profile.announce == "SELF" then
			DEFAULT_CHAT_FRAME:AddMessage(L["we_have"]..count..L["fails_on"]..players..L["diff..."])
		else
			SendChatMessage(L["we_have"]..count..L["fails_on"]..players..L["diff..."],target, nil, whisper)
		end
		local b,amount=1,addon.db.profile.stats_value
		if addon.db.profile.stats_all then
			amount = #sorttable
		end		
		for k,v in ipairs(sorttable) do
			if (b <= amount) then
				if b == 1 then
					if stat == "ostats" then
						if addon.db.profile.announce == "SELF" then
							DEFAULT_CHAT_FRAME:AddMessage(L["admiral"]..v[1]..L["failed"]..v[2].."x ("..string.format("%02.1f",v[2]/count*100).."%)")
						else
							SendChatMessage(L["admiral"]..v[1]..L["failed"]..v[2].."x ("..string.format("%02.1f",v[2]/count*100).."%)",target, nil, whisper)
						end
					else
						if addon.db.profile.announce == "SELF" then
							DEFAULT_CHAT_FRAME:AddMessage(L["captain"]..v[1]..L["failed"]..v[2].."x ("..string.format("%02.1f",v[2]/count*100).."%)")
						else
							SendChatMessage(L["captain"]..v[1]..L["failed"]..v[2].."x ("..string.format("%02.1f",v[2]/count*100).."%)",target, nil, whisper)
						end
					end
				else
					if addon.db.profile.announce == "SELF" then
						DEFAULT_CHAT_FRAME:AddMessage(b..". "..v[1]..L["failed"]..v[2].."x ("..string.format("%02.1f",v[2]/count*100).."%)")
					else
						SendChatMessage(b..". "..v[1]..L["failed"]..v[2].."x ("..string.format("%02.1f",v[2]/count*100).."%)",target, nil, whisper)
					end
				end
			end
			b=b+1
		end
		if (GetNumPartyMembers() > 0) or (GetNumRaidMembers() > 0) then
			local r,name={},""
			if (GetNumRaidMembers() > 0) then
				for i=1,GetNumRaidMembers() do
					name=GetRaidRosterInfo(i)
					if (Count[name] == nil) then
						table.insert(r,name)
					end
				end
			else
				for i=1,GetNumPartyMembers() do
					name = UnitName("party"..i)
					if (Count[name] == nil) then
						table.insert(r,name)
					end
				end
				name = UnitName("player")
				if (Count[name] == nil) then
					table.insert(r,name)
				end
			end
			SendChatMessage(L["didnt_fail"]..table.concat(r,", "),target, nil, whisper)
		end
	end	
end

local debug = false
--[===[@debug@ 
function addon:Debug()
	if debug then
		debug = false
	else
		debug = true
	end
end	
--@end-debug@]===]
function addon:CombatLog(event,...)
	local timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags = select(1,...)
		
	if (sourceName=="Mirror Image") or (destName=="Mirror Image") then
		return;
	end
--------------------------------------------------------------------------------
--------------------	DEBUG EVENTS	----------------------------------------
--------------------------------------------------------------------------------

if debug then

	if timestamp then
		temp_stamp = tostring(timestamp)
		if temp_stamp:find(":") then      
			local hour = temp_stamp:sub(1,2)
			local minute = temp_stamp:sub(4,5) 
			local sec = temp_stamp:sub(7,8)
			local milisec = temp_stamp:sub(9)
			timestamp = tonumber("10000"..hour*3600+minute*60+sec..milisec)
		end
	end

	if (type=="SPELL_AURA_APPLIED") then
		local spellId = select(9,...)		
		if ((spellId==19879) or (spellId==19880)) then
			print("debug1")
			addon:AddFail(destName,"blabla",spellId)
		elseif spellId==696 or spellId==28189 then
			addon:AddFail(destName,"blabla",spellId)
		end
	end

	if (type=="SPELL_CAST_START") then
		local spellId = select(9,...)
		if (spellId==635) then
			print("debug2")
			addon:AddFail(sourceName,L["shaman_healing"],635)
		end
	end
	
	if type=="SPELL_CAST_SUCCESS" then
		local spellId = select(9,...)
		if (spellId==20154) then
			addon:AddFail(sourceName,"SEAL",20154)
		end
	end
	

	if (type=="SPELL_CAST_START") then
		local spellId = select(9,...)
		if ((spellId==49273) or (spellId==49276)) then
			addon:AddFail(sourceName,L["shaman_healing"],spellId)
		end
	end

	if (type=="SPELL_CAST_START") then
		local spellId = select(9,...)
		if (spellId==55459) then
			addon:AddFail(sourceName,L["moving"].."( "..L["Frost Bomb"]..")",spellId)
		end
	end

	if (type=="SPELL_DAMAGE") then
		local spellId = select(9,...)
		if (spellId=="Earth Shock") then
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				addon:AddFail(destName,"ETest",spellId)
			end
		end
	end

	if (type=="SPELL_DAMAGE") then
		local spellId = select(9,...)
		if ((spellId==49236) or (spellId==49231)) then
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				addon:AddFail(destName,"FTest",spellId)
			end
		end
	end
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

if (type=="SPELL_DAMAGE") then
	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
		local spellId,_,_,amount,overkill = select(9,...)
---- FLAME LEVIATHAN ----		
---- RAZORSCALE ----		
--6/3 13:34:02.937  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x0300000001720FE7,"Methoz",0x514,64733,"Devouring Flame",0x4,8751,0,4,0,0,0,nil,nil,nil
--6/8 19:44:46.984  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x0300000001D32634,"Aryu",0x514,64704,"Devouring Flame",0x4,819,0,4,1157,0,3948,nil,nil,nil
-- Devouring Flame -- Razorscale
		if ((spellId==64733) or (spellId==64704)) then	
			if addon.db.profile.devouring_flame then
				if not addon.db.profile.noexceptions then
					if UnitHealthMax(destName) <= 35000 then
						if LastEvent1[destName] == nil then
							LastEvent1[destName] = timestamp
							addon:AddFail(destName,L["moving"],spellId)
						else
							if (timestamp - LastEvent1[destName]) > 4 then
								addon:AddFail(destName,L["moving"],spellId)
							end
							LastEvent1[destName] = timestamp
						end
					end
				else 
					if LastEvent1[destName] == nil then
						LastEvent1[destName] = timestamp
						addon:AddFail(destName,L["moving"],spellId)
					else
						if (timestamp - LastEvent1[destName]) > 4 then
							addon:AddFail(destName,L["moving"],spellId)
						end
						LastEvent1[destName] = timestamp
					end
				end
			end
---- XT-002 ----			
--5/7 18:19:04.078  SPELL_DAMAGE,0xF1300084D1002E23,"Void Zone",0xa48,0x0300000001D3239F,"Diomache",0x514,64208,"Consumption",0x20,6258,0,32,0,0,0,nil,nil,nil
-- Void Zone -- XT-002			
		elseif ((spellId==64208) or (spellId==64206)) then
			if addon.db.profile.void_zone then
				addon:AddFail(destName,L["moving"],spellId)
			end
--6/10 19:52:16.250  SPELL_DAMAGE,0x0300000001D2BD79,"Korta",0x8000514,0x0300000001D2A5F8,"Hams",0x514,65120,"Searing Light",0x42,2400,0,66,0,0,600,nil,nil,nil
-- Searing Light (Light Bomb) -- XT-002			
		elseif (spellId==65120) then
			if addon.db.profile.lightbomb then
				if sourceName ~= destName then
					if searing_applied and ((timestamp - searing_applied) > addon.db.profile.lightbomb_value) then
						if LastEvent1[sourceName] == nil then
							LastEvent1[sourceName] = timestamp
							addon:AddFail(sourceName,L["moving"],spellId)
						else
							if (timestamp - LastEvent1[sourceName]) > 4 then
								EnsidiaFails:AddFail(sourceName,L["moving"],spellId)
							end
							LastEvent1[sourceName] = timestamp
						end
					end
				end		
			end
--6/10 19:57:57.484  SPELL_DAMAGE,0x0300000001D63C15,"Clozern",0x2000514,0x0300000001D2A633,"Krematoria",0x514,64233,"Gravity Bomb",0x20,13483,0,32,1498,0,0,nil,nil,nil
-- Gravity Bomb -- XT-002			
		elseif (spellId==64233) then
			if addon.db.profile.gravity_bomb then
				if sourceName ~= destName then
					if ChargeCounter[sourceName] == nil then
						ChargeCounter[sourceName] = 1
						LastEvent1[sourceName] = timestamp
					elseif (timestamp - LastEvent1[sourceName]) < 1 then
						ChargeCounter[sourceName] = ChargeCounter[sourceName] + 1
						LastEvent1[sourceName] = timestamp
					else
						ChargeCounter[sourceName] = 1
						LastEvent1[sourceName] = timestamp
					end
					if ChargeCounter[sourceName] >= 2 then
						if not FailedatGravityBomb then
							FailedatGravityBomb = true
							addon:AddFail(sourceName,L["moving"],spellId)
							ChargeCounter[sourceName] = 0						
						else
							addon:ScheduleTimer("GravityBomb", self.GravityBomb, 2)
						end
					end		
				end
			end
---- THE IRON COUNCIL ----			
--3/20 19:22:43.389  SPELL_DAMAGE,0xF130008059000703,"Stormcaller Brundir",0x8000a48,0x01000000007C5537,"Vanen",0x512,61878,"Overload",0x8,17460,0,8,2000,0,0,nil,nil,nil
-- Overload -- The Iron Council - small guy			
		elseif (((spellId==61878) or (spellId==63480))and sourceName==BB["Stormcaller Brundir"]) then
			if addon.db.profile.overload then
				if amount >= addon.db.profile.overload_value then
					addon:AddFail(destName,L["moving"],spellId)
				end			
			end
---- KOLOGARN ----
-- Focused Eyebeam -- Kologarn
		elseif ((spellId==63976) or (spellId==63346)) then		
			if addon.db.profile.eyebeam then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					addon:AddFail(destName,L["moving"],spellId)
				else
					if (timestamp - LastEvent1[destName]) > 5 then
						addon:AddFail(destName,L["moving"],spellId)
					end
					LastEvent1[destName] = timestamp
				end
			end
---- AURIAYA ----
-- Seeping Feral Essence -- Auriaya			
		elseif ((spellId==64459) or (spellId==64675)) then
			if addon.db.profile.seeping_feral_essence then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					addon:AddFail(destName,L["moving"],spellId)
				else
					if (timestamp - LastEvent1[destName]) > 4 then
						addon:AddFail(destName,L["moving"],spellId)
					end
					LastEvent1[destName] = timestamp
				end	
			end
--6/8 20:29:47.625  SPELL_DAMAGE,0xF1300082EB000853,"Auriaya",0x8010a48,0x0300000002289DD9,"Spritzor",0x514,64422,"Sonic Screech",0x1,7263,0,1,0,0,0,nil,nil,nil
-- Sonic Screech -- Auriaya
		elseif ((spellId==64688) or (spellId==64422)) then
			if addon.db.profile.sonic_screech then
				not_failed_on_sonic[not_failed_on_sonic_counter] = destName		
				not_failed_on_sonic_counter = not_failed_on_sonic_counter + 1
			end
---- HODIR ----		
--5/27 16:15:53.375  SPELL_DAMAGE,0xF130008191003903,"Icicle",0xa48,0x0300000001D2A633,"Krematoria",0x514,62457,"Ice Shards",0x10,12600,0,16,1400,0,0,nil,nil,nil			
--5/27 16:16:01.062  SPELL_DAMAGE,0xF13000819500390B,"Snowpacked Icicle",0xa48,0x03000000000BE9BC,"Tun",0x514,65370,"Ice Shards",0x10,7840,0,16,2800,0,0,nil,nil,nil
-- Ice Shards -- Hodir
		elseif ((spellId==62457) or (spellId==65370)) then
			if addon.db.profile.ice_shards then
				addon:AddFail(destName,L["moving"],spellId)		
			end
---- THORIM ----			
--4/16 18:20:24.295  SPELL_DAMAGE,0xF130008061018374,"Thorim",0x8010a48,0x0500000001E8AF39,"Thefeint",0x514,62466,"Lightning Charge",0x8,8977,0,8,3966,0,0,nil,nil,nil
-- Lightning Charge -- Thorim			
		elseif ((spellId==62466) and sourceName==BB["Thorim"]) then
			if addon.db.profile.lightning_charge then
				addon:AddFail(destName,L["moving"],spellId)
			end
-- Chain Lightning -- Thorim	
--6/8 20:48:43.937  SPELL_DAMAGE,0xF130008061000BF9,"Thorim",0x10a48,0x0300000001D2A5F8,"Hams",0x514,64390,"Chain Lightning",0x8,3420,0,8,528,0,1140,nil,nil,nil		
--6/10 21:38:21.421  SPELL_DAMAGE,0xF130008061003E24,"Thorim",0x10a48,0x03000000003E714B,"Buzzkill",0x514,62131,"Chain Lightning",0x8,3720,0,8,1162,0,930,nil,nil,nil
		elseif ((spellId==64390) or (spellId==62131)) then		
			if addon.db.profile.chainlightning then
				if amount > 10000 then -- at least 3 ppl hugging, and I think highest damage appears in combat log first always
					three_people_hugging = true
					addon:ScheduleTimer("ChainLightningCD", self.ChainLightningCD, 3)
				end
				if three_people_hugging then
					if LastEvent3 == nil then
						LastEvent3 = timestamp
					else
						if (timestamp - LastEvent3) < 1 then
							addon:AddFail(destName,L["moving"],spellId)
						end
						LastEvent3 = timestamp
					end	
				end
			end
--4/16 01:06:26.414  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x05000000027ECA9C,"Cn",0x514,62465,"Runic Smash",0x4,6544,0,4,3116,0,0,nil,nil,nil
-- Runic Smash -- Thorim tunnel			
		elseif (spellId==62465) then
			if addon.db.profile.runic_smash then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					addon:AddFail(destName,L["left and right"],spellId)
				else
					if (timestamp - LastEvent1[destName]) > 4 then
						addon:AddFail(destName,L["left and right"],spellId)
					end
					LastEvent1[destName] = timestamp
				end		
			end
---- FREYA ----
--6/4 21:29:59.890  SPELL_DAMAGE,0xF1300085510747B6,"Nature Bomb",0xa48,0xF14052F12D001499,"Flaadhun",0x1114,64650,"Nature Bomb",0x8,1140,0,8,2651,0,0,nil,nil,nil
--6/8 21:09:18.656  SPELL_DAMAGE,0xF130008551008B28,"Nature Bomb",0xa48,0xF14052F1300001AD,"Zortip",0x1114,64587,"Nature Bomb",0x8,939,0,8,1175,0,0,nil,nil,nil
-- Nature Bomb -- Freya
		elseif ((spellId==64650) or (spellId==64587)) then
			if addon.db.profile.nature_bomb then
				local roots = GetSpellInfo(62861)
				local debuff_name = UnitDebuff(destName, roots)			
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					if debuff_name ~= roots then
						addon:AddFail(destName,L["moving"],spellId)
					end
				else
					if (timestamp - LastEvent1[destName]) > 2 then
						if debuff_name ~= roots then
							addon:AddFail(destName,L["moving"],spellId)
						end
					end
					LastEvent1[destName] = timestamp
				end
			end
---- MIMIRON ----			
-- Flames -- Mimiron		
--6/4 22:29:27.046  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x0300000001D2A4D6,"Mackzter",0x514,64566,"Flames",0x4,6488,0,4,2958,0,0,nil,nil,nil	
		elseif (spellId==64566) then	
			if addon.db.profile.flames then
				if overkill > 0 then
					addon:AddFail(destName,L["moving"],spellId)
				end
			end
--6/4 22:35:48.921  SPELL_DAMAGE,0xF13000856307C937,"Emergency Fire Bot",0xa48,0x0300000001D2BD79,"Korta",0x514,64619,"Water Spray",0x10,14799,0,16,4111,0,0,nil,nil,nil
-- Water Spray -- Mimiron
		elseif (spellId==64619) then -- 10 man spell Id?
			if addon.db.profile.water_spray then
				addon:AddFail(destName,L["moving"],spellId)
			end		
--4/16 13:16:23.167  SPELL_DAMAGE,0xF13000863A00929D,"Proximity Mine",0xa48,0x05000000027ECCF4,"Hotalicious",0x512,63009,"Explosion",0x4,22500,16608,4,2500,0,0,nil,nil,nil
--6/8 21:20:02.390  SPELL_DAMAGE,0xF13000863A009377,"Proximity Mine",0xa48,0x03000000000B526B,"Kungen",0x514,66351,"Explosion",0x4,5330,0,4,2700,0,0,nil,nil,nil
-- Proxmity Mine -- Mimiron			
		elseif (((spellId==63009) or (spellId==66351)) and (sourceName==L["Proximity Mine"])) then	
			if addon.db.profile.mines then
				if addon.db.profile.noexceptions == false then
					if UnitHealthMax(destName) <= 35000 then
						if LastEvent1[destName] == nil then
							LastEvent1[destName] = timestamp
							addon:AddFail(destName,L["moving"],63027)
						else
							if (timestamp - LastEvent1[destName]) > 2 then
								addon:AddFail(destName,L["moving"],63027)
							end
							LastEvent1[destName] = timestamp
						end	
					end
				else 
					if LastEvent1[destName] == nil then
						LastEvent1[destName] = timestamp
						addon:AddFail(destName,L["moving"],63027)
					else
						if (timestamp - LastEvent1[destName]) > 2 then
							addon:AddFail(destName,L["moving"],63027)
						end
						LastEvent1[destName] = timestamp
					end						
				end
			end
--3/13 20:56:28.111  SPELL_DAMAGE,0xF1300084FF001E10,"Rocket Strike",0xa48,0x01000000007C088D,"Veev",0x511,63041,"Rocket Strike",0x4,676800,657890,4,200000,0,0,nil,nil,nil
-- Rocket Strike -- Mimiron
		elseif (spellId==63041) then 
			if addon.db.profile.rocket_strike then
				addon:AddFail(destName,L["moving"],spellId)
			end
--3/13 21:05:11.205  SPELL_DAMAGE,0xF1500083730020DC,"VX-001",0x10a48,0x01000000007F4785,"Lawlpurge",0x514,63293,"P3Wx2 Laser Barrage",0x40,19400,1821,64,0,0,0,nil,nil,nil
-- Laser Barrage -- Mimiron			
		elseif (spellId==63293) then
			if addon.db.profile.laser_barrage then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					addon:AddFail(destName,L["moving"],spellId)
				else
					if (timestamp - LastEvent1[destName]) > 3 then
						addon:AddFail(destName,L["moving"],spellId)
					end
					LastEvent1[destName] = timestamp
				end	
			end
--3/13 21:17:23.756  SPELL_DAMAGE,0xF150008298002210,"Leviathan Mk II",0x10a48,0xF1300007AC0025A9,"Treant",0x1114,63631,"Shock Blast",0x8,97000,92908,8,0,0,0,nil,nil,nil
-- Shock Blast -- Mimiron			
		elseif (spellId==63631) then
			if addon.db.profile.shock_blast then
				addon:AddFail(destName,L["moving"],spellId)
			end
--4/25 12:58:04.640  SPELL_DAMAGE,0xF1300085650042DC,"Frost Bomb",0xa48,0x030000000106ED69,"Pachinko",0x514,65333,"Explosion",0x10,39839,35289,16,9960,0,0,nil,nil,nil	
--6/22 21:41:50.062  SPELL_DAMAGE,0xF130008565028F92,"Frost Bomb",0xa48,0x0300000001D2D53E,"Arinen",0x514,64626,"Explosion",0x10,8983,0,16,998,0,0,nil,nil,nil
-- Frost Bomb -- Mimiron			
		elseif (sourceName==L["Frost Bomb"]) and ((spellId==65333) or (spellId==64626))  then
			if addon.db.profile.frost_bomb then
				addon:AddFail(destName,L["moving"],64623)
			end
--4/16 13:35:12.750  SPELL_DAMAGE,0xF13000842C0094E3,"Bomb Bot",0xa48,0x05000000027ECCA5,"Naddia",0x512,63801,"Bomb Bot",0x4,20216,4025,4,5054,0,0,nil,nil,nil
-- Bomb Bot -- Inside Mimiron's room			
		elseif (spellId==63801) then
			if addon.db.profile.bombbots then
				if addon.db.profile.noexceptions == false then
					if UnitHealthMax(destName) <= 35000 then
						if (GetMinimapZoneText()==BB["The Spark of Imagination"]) then			
							addon:AddFail(destName,L["moving"],spellId)
						end
					end
				else
					if (GetMinimapZoneText()==L["The Spark of Imagination"]) then			
						addon:AddFail(destName,L["moving"],spellId)
					end				
				end
			end
---- GENERAL VEZAX ----	
--4/16 22:06:17.885  SPELL_DAMAGE,0xF1300081F702D928,"General Vezax",0x10a48,0x05000000027FCDFE,"Kosie",0x514,62659,"Shadow Crash",0x20,9413,0,32,2285,0,0,nil,nil,nil
-- Shadow Crash -- General Vezax			
		elseif ((spellId==63277) or (spellId==62659)) and sourceName==BB["General Vezax"] then
			if addon.db.profile.shadow_crash then
				addon:AddFail(destName,L["moving"],spellId)	
			end
--4/29 22:38:40.135  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x0500000002BFA226,"Cimsir",0x514,63338,"Saronite Vapors",0x20,20480,8387,32,5120,0,0,nil,nil,nil
--	Saronite Vapors  -- General Vezax			
		elseif (spellId==63338) then	
			if addon.db.profile.saronite_vapor then
				if overkill > 0 then
					EnsidiaFails:AddFail(destName,L["moving"],spellId)
				end
			end			
---- YOGG-SARON ----			
--Line 435447 : 4/17 19:50:51.493  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x05000000027ECAFC,"Turyia",0x514,63884,"Death Ray",0x8,12800,3981,8,4000,0,0,nil,nil,nil		
-- Death Ray -- Yogg-Saron			
		elseif ((spellId==63891) or (spellId==63884)) then
			if addon.db.profile.death_ray then
				local malady = GetSpellInfo(63830)
				local debuff_name = UnitDebuff(destName, malady)
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					if debuff_name ~= malady then		
						addon:AddFail(destName,L["moving"],spellId)
					end
				else
					if (timestamp - LastEvent1[destName]) > 3 then
						if debuff_name ~= malady then		
							addon:AddFail(destName,L["moving"],spellId)
						end
					end
					LastEvent1[destName] = timestamp
				end	
			end
--5/17 19:28:01.718  SPELL_DAMAGE,0xF150008208022230,"Yogg-Saron",0xa48,0x0300000001D2A511,"Zemme",0x514,64164,"Lunatic Gaze",0x20,5592,0,32,621,0,0,nil,nil,nil
--5/17 19:27:08.140  SPELL_DAMAGE,0xF1300084C6024673,"Laughing Skull",0xa48,0x0300000002171BAB,"Fenita",0x514,64168,"Lunatic Gaze",0x20,1120,0,32,350,0,0,nil,nil,nil
-- Lunatic Gaze -- Yogg Saron
		elseif ((spellId==64168) or (spellId==64164)) then
			if addon.db.profile.lunatic_gaze then
				if LunaticGaze[destName] == nil then
					LunaticGaze[destName] = 1
				else
					if (spellId==64168) then
						LunaticGaze[destName] = LunaticGaze[destName] + 1			
					elseif (spellId==64164) then
						LunaticGaze[destName] = LunaticGaze[destName] + 2
					end
				end
				if LunaticGaze[destName] >= 15 then
					LunaticGaze[destName] = 0
					addon:AddFail(destName,L["turning away"],64164)
				end
			end
---- ALGALON ----			
--5/5 17:23:21.435  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x0300000001B85A77,"Ragebar",0x512,62311,"Cosmic Smash",0x4,5262,0,4,0,0,0,nil,nil,nil
-- Cosmic Smash -- Algalon
		elseif ((spellId==62311) or (spellId==64596)) then
			if addon.db.profile.cosmic_smash then
				if amount >= addon.db.profile.cosmic_smash_value then
					if LastEvent1[destName] == nil then
						LastEvent1[destName] = timestamp
						addon:AddFail(destName,L["moving"],spellId)
					else
						if (timestamp - LastEvent1[destName]) > 2 then
							addon:AddFail(destName,L["moving"],spellId)
						end
						LastEvent1[destName] = timestamp
					end	
				end
			end
-- Big Bang -- Algalon			
		elseif ((spellId==64584) or (spellId==64443)) then
			if addon.db.profile.big_bang then
				if overkill > 0 then
					addon:AddFail(destName,L["moving"],spellId)
				end		
			end
		end
	end
end

if (type=="SPELL_MISSED") or (type=="SPELL_DAMAGE") then
	local spellId = select(9,...)
	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
--6/8 20:57:56.781  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x0300000002289DD9,"Spritzor",0x512,62590,"Nature's Fury",0x8,4272,0,8,0,0,0,nil,nil,nil			
--6/4 20:54:31.468  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0xF14052F12D001360,"Flaadhun",0x1114,63570,"Nature's Fury",0x8,1577,0,8,951,0,0,nil,nil,nil
-- Nature's Fury -- Freya		
		if ((spellId==62590) or (spellId==63570)) then
			if addon.db.profile.natures_fury then
				local roots = GetSpellInfo(62861)
				local debuff_name, debuff_name1, debuff_name2 = nil, nil, nil
				if fury_on then
					debuff_name = UnitDebuff(fury_on, roots)		
				elseif fury_on1 then
					debuff_name1 = UnitDebuff(fury_on1, roots)	
				elseif fury_on2 then
					debuff_name2 = UnitDebuff(fury_on2, roots)	
				end	
				
				if (destName == fury_on) then			-- 1st guy is taking damage
					if LastEvent1[fury_on] then				-- someone already took damage 								
						if (timestamp - LastEvent1[fury_on]) < addon.db.profile.natures_fury_combatlog_time then 	--someone took damage from the 1st guy just before he took damage					
							if LastEvent2[fury_on] == nil then		-- 1st time the 1st guy takes damage							
								LastEvent2[fury_on] = timestamp
							else							
								if (timestamp - LastEvent2[fury_on]) > addon.db.profile.natures_fury_move_time or (timestamp - ChargeCounter[fury_on]) > addon.db.profile.natures_fury_move_time then	-- 1st guy had at least 3 sec to move							
									if debuff_name ~= roots then
										addon:AddFail(fury_on,L["moving"],spellId) 
										ChargeCounter[fury_on] = timestamp
									end							
								end												
							end	
						end				
					end			
					LastEvent2[fury_on] = timestamp				
				else			-- not the 1st guy is taking damage			
					if fury_on then
						if LastEvent1[fury_on] == nil then  -- 1st time someone else took damage					
							LastEvent1[fury_on] = timestamp
						else					-- not the 1st guy and not the 1st time someone else took damage				
							if LastEvent2[fury_on] then		-- 1st guy already took damage						
								if (timestamp - LastEvent2[fury_on]) < addon.db.profile.natures_fury_combatlog_time then -- 1st guy took took damage just before he took damage						
									if (timestamp - ChargeCounter[fury_on]) > addon.db.profile.natures_fury_move_time then -- antispam, someone took damage, and it was 3 sec ago that 1st guy damaged 1 guy
										if debuff_name ~= roots then
											addon:AddFail(fury_on,L["moving"],spellId)
											ChargeCounter[fury_on] = LastEvent2[fury_on] -- store the last time this happened
										end								
									end	
								end
							end
							LastEvent1[fury_on] = timestamp -- regardless if the 1st guy took damage or not, store when someone else took damage
						end
					end
				end			
				
				if (destName == fury_on1) then			-- 2nd guy is taking damage
					if LastEvent1[fury_on1] then				-- someone already took damage 												
						if (timestamp - LastEvent1[fury_on1]) < addon.db.profile.natures_fury_combatlog_time then 	--someone took damage from the 2nd guy just before he took damage										
							if LastEvent2[fury_on1] == nil then		-- 1st time the 2nd guy takes damage							
								LastEvent2[fury_on1] = timestamp
							else
								if (timestamp - LastEvent2[fury_on1]) > addon.db.profile.natures_fury_move_time or (timestamp - ChargeCounter[fury_on1]) > addon.db.profile.natures_fury_move_time then	-- 2nd guy had at least 3 sec to move							
									if debuff_name1 ~= roots then
										addon:AddFail(fury_on1,L["moving"],spellId) 
										ChargeCounter[fury_on1] = timestamp
									end							
								end												
							end	
						end				
					end			
					LastEvent2[fury_on1] = timestamp				
				else			-- not the 2nd guy is taking damage					
					if fury_on1 then
						
						if LastEvent1[fury_on1] == nil then  -- 1st time someone else took damage					
							LastEvent1[fury_on1] = timestamp
						else					-- not the 2nd guy and not the 1st time someone else took damage				
							if LastEvent2[fury_on1] then		-- 2nd guy already took damage						
								if (timestamp - LastEvent2[fury_on1]) < addon.db.profile.natures_fury_combatlog_time then -- 2nd guy took took damage just before he took damage						
									if (timestamp - ChargeCounter[fury_on1]) > addon.db.profile.natures_fury_move_time then -- antispam, someone took damage, and it was 3 sec ago that 2nd guy damaged 1 guy
										if debuff_name1 ~= roots then
											addon:AddFail(fury_on1,L["moving"],spellId)
											ChargeCounter[fury_on1] = LastEvent2[fury_on1] -- store the last time this happened
										end							
									end	
								end
							end
							LastEvent1[fury_on1] = timestamp -- regardless if the 2nd guy took damage or not, store when someone else took damage						
						end	
					end
				end

				if (destName == fury_on2) then			-- 3rd guy is taking damage
					if LastEvent1[fury_on2] then				-- someone already took damage 												
						if (timestamp - LastEvent1[fury_on2]) < addon.db.profile.natures_fury_combatlog_time then 	--someone took damage from the 3rd guy just before he took damage										
							if LastEvent2[fury_on2] == nil then		-- 1st time the 3rd guy takes damage							
								LastEvent2[fury_on2] = timestamp
							else
								if (timestamp - LastEvent2[fury_on2]) > addon.db.profile.natures_fury_move_time or (timestamp - ChargeCounter[fury_on2]) > addon.db.profile.natures_fury_move_time then	-- 3rd guy had at least 3 sec to move							
									if debuff_name2 ~= roots then
										addon:AddFail(fury_on2,L["moving"],spellId) 
										ChargeCounter[fury_on2] = timestamp
									end							
								end												
							end	
						end				
					end			
					LastEvent2[fury_on2] = timestamp				
				else			-- not the 3rd guy is taking damage					
					if fury_on2 then
						
						if LastEvent1[fury_on2] == nil then  -- 1st time someone else took damage					
							LastEvent1[fury_on2] = timestamp
						else					-- not the 3rd guy and not the 1st time someone else took damage				
							if LastEvent2[fury_on2] then		-- 3rd guy already took damage						
								if (timestamp - LastEvent2[fury_on2]) < addon.db.profile.natures_fury_combatlog_time then -- 3rd guy took took damage just before he took damage						
									if (timestamp - ChargeCounter[fury_on2]) > addon.db.profile.natures_fury_move_time then -- antispam, someone took damage, and it was 3 sec ago that 3rd guy damaged 1 guy
										if debuff_name2 ~= roots then
											addon:AddFail(fury_on2,L["moving"],spellId)
											ChargeCounter[fury_on2] = LastEvent2[fury_on2] -- store the last time this happened
										end							
									end	
								end
							end
							LastEvent1[fury_on2] = timestamp -- regardless if the 3rd guy took damage or not, store when someone else took damage						
						end	
					end
				end				
			end
		end
	end
end

if (type=="SPELL_AURA_APPLIED_DOSE") then
	local spellId,_,_,_,stack = select(9,...)
---- FREYA ----
--6/10 21:54:38.953  SPELL_AURA_APPLIED_DOSE,0xF1300080940356BC,"Snaplasher",0xa48,0xF1300080940356BC,"Snaplasher",0xa48,64190,"Hardened Bark",0x1,BUFF,84
--6/8 20:56:06.781  SPELL_AURA_APPLIED_DOSE,0xF130008094007DBD,"Snaplasher",0xa48,0xF130008094007DBD,"Snaplasher",0xa48,62663,"Hardened Bark",0x1,BUFF,2	
	if ((spellId == 64190) or (spellId==62663)) and stack > addon.db.profile.hardened_bark_value then				
		if addon.db.profile.hardened_bark then
			hardened_bark = true
		end
---- YOGG-SARON ----
--6/8 21:59:13.015  SPELL_AURA_APPLIED_DOSE,0xF1300084AE00BD56,"Crusher Tentacle",0xa48,0xF1300084AE00BD56,"Crusher Tentacle",0xa48,57689,"Focused Anger",0x1,BUFF,3		
--6/15 21:05:29.765  SPELL_AURA_APPLIED_DOSE,0xF1300084AE000650,"Crusher Tentacle",0xa48,0xF1300084AE000650,"Crusher Tentacle",0xa48,57689,"Focused Anger",0x1,BUFF,6	
	elseif (spellId==57689) and stack > addon.db.profile.focused_anger_value then
		if addon.db.profile.focused_anger then
			focused_anger = true
		end
	end	
end

if (type=="SPELL_DAMAGE") or (type=="SWING_DAMAGE") then
local spellId = select(9,...)
	if destName == 	L["Snaplasher"] then			
		if hardened_bark and addon.db.profile.hardened_bark then
			if bit.band(sourceFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				if LastEvent1[sourceName] == nil then
					LastEvent1[sourceName] = timestamp
					if type=="SWING_DAMAGE" then
						addon:AddFail(sourceName,GetSpellInfo(64190).." ("..GetSpellInfo(5547)..")",64190)
					else
						addon:AddFail(sourceName,GetSpellInfo(64190).." ("..GetSpellInfo(spellId)..")",64190)
					end
				else
					if (timestamp - LastEvent1[sourceName]) > 3 then
						if type=="SWING_DAMAGE" then
							addon:AddFail(sourceName,GetSpellInfo(64190).." ("..GetSpellInfo(5547)..")",64190)
						else
							addon:AddFail(sourceName,GetSpellInfo(64190).." ("..GetSpellInfo(spellId)..")",64190)
						end
					end
					LastEvent1[sourceName] = timestamp
				end		
			else
				num = GetNumRaidMembers()
				local name = nil
				for i=1 ,num do
					if sourceName == UnitName("raidpet"..i) then
						name = UnitName("raid"..i)						
					end      
				end
				if name then
					if LastEvent1[sourceName] == nil then
						LastEvent1[sourceName] = timestamp
						addon:AddFail(name,GetSpellInfo(64190).." (pet)",64190)
					else
						if (timestamp - LastEvent1[sourceName]) > 3 then
							addon:AddFail(name,GetSpellInfo(64190).." (pet)",64190)
						end
						LastEvent1[sourceName] = timestamp
					end	
				end
			end
		end
	elseif destName == L["Crusher Tentacle"] then
		if focused_anger and addon.db.profile.focused_anger and (UnitBuff(destName,GetSpellInfo(57689))==nil) then
			if bit.band(sourceFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				if LastEvent1[sourceName] == nil then
					LastEvent1[sourceName] = timestamp
					if type=="SWING_DAMAGE" then
						addon:AddFail(sourceName,GetSpellInfo(57689).." ("..GetSpellInfo(5547)..")",57689)
					else
						addon:AddFail(sourceName,GetSpellInfo(57689).." ("..GetSpellInfo(spellId)..")",57689)
					end
				else
					if (timestamp - LastEvent1[sourceName]) > 3 then
						if type=="SWING_DAMAGE" then
							addon:AddFail(sourceName,GetSpellInfo(57689).." ("..GetSpellInfo(5547)..")",57689)
						else
							addon:AddFail(sourceName,GetSpellInfo(57689).." ("..GetSpellInfo(spellId)..")",57689)
						end
					end
					LastEvent1[sourceName] = timestamp
				end				
			end				
		end
	end
end

if (type=="SPELL_MISSED") then
	local spellId = select(9,...)
	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
		if ((spellId==64688) or (spellId==64422)) then
			if addon.db.profile.sonic_screech then
				not_failed_on_sonic[not_failed_on_sonic_counter] = destName		
				not_failed_on_sonic_counter = not_failed_on_sonic_counter + 1
			end
		end
	end
end

if (type=="SPELL_PERIODIC_DAMAGE") then
	local spellId = select(9,...)
	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
--5/27 16:58:34.546  SPELL_PERIODIC_DAMAGE,0xF130008192005B71,"Sun Beam",0xa48,0x030000000147252F,"Poptisse",0x200514,62865,"Unstable Energy",0x8,7459,0,8,854,0,0,nil,nil,nil
--6/8 20:57:55.765  SPELL_PERIODIC_DAMAGE,0xF130008192007FE4,"Sun Beam",0xa48,0xF130007250007FCE,"Spirit Wolf",0x1112,62451,"Unstable Energy",0x8,1177,0,8,0,0,0,nil,nil,nil
-- Unstable Energy -- Freya
		if ((spellId==62865) or (spellId==62451)) then
			if addon.db.profile.unstable_energy then
				local roots = GetSpellInfo(62861)
				local debuff_name = UnitDebuff(destName, roots)				
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					if debuff_name ~= roots then
						addon:AddFail(destName,L["moving"],spellId)
					end
				else
					if (timestamp - LastEvent1[destName]) > 4 then
						if debuff_name ~= roots then
							addon:AddFail(destName,L["moving"],spellId)
						end
					end
					LastEvent1[destName] = timestamp
				end
			end
--6/4 20:11:41.531  SPELL_PERIODIC_DAMAGE,0x0000000000000000,nil,0x80000000,0x0300000001D2D545,"Meka",0x8000514,62602,"Blizzard",0x10,4976,0,16,553,0,0,nil,nil,nil			
--6/8 20:49:13.859  SPELL_PERIODIC_DAMAGE,0x0000000000000000,nil,0x80000000,0xF14052F130000094,"Zortip",0x1114,62576,"Blizzard",0x10,339,0,16,727,0,0,nil,nil,nil
-- Blizzard -- Thorim
		elseif ((spellId==62576) or (spellId==62602)) then
			if addon.db.profile.blizzard then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
				else
					if (timestamp - LastEvent1[destName]) > 4 then
						addon:AddFail(destName,L["moving"],spellId)
						LastEvent1[destName] = nil
					end
				end	
			end
--6/10 20:33:18.687  SPELL_PERIODIC_DAMAGE,0xF13000809F003A34,"Runemaster Molgeim",0x10a48,0x0300000001D32634,"Aryu",0x514,63490,"Rune of Death",0x20,2884,0,32,700,0,0,nil,nil,nil
-- Rune of Death -- The Iron Council			
		elseif ((spellId==63490) or (spellId==62269))then
			if addon.db.profile.rune_of_death then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
				else
					if (timestamp - LastEvent1[destName]) > addon.db.profile.rune_of_death_value and (timestamp - LastEvent1[destName]) < 20 then
						if LastEvent2[destName] == nil then
							LastEvent2[destName] = timestamp
							addon:AddFail(destName,L["moving"],spellId)
						else
							if (timestamp - LastEvent2[destName]) > (addon.db.profile.rune_of_death_value*2) then
								addon:AddFail(destName,L["moving"],spellId)
								LastEvent2[destName] = timestamp
							end
						end
						LastEvent1[destName] = nil
					end
				end	
			end
		end
	end
end

if (type=="SPELL_ENERGIZE") then
	local spellId,_,_,amount = select(9,...)
	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
--4/29 22:38:40.135  SPELL_ENERGIZE,0x0000000000000000,nil,0x80000000,0x0500000002BFA226,"Cimsir",0x514,63337,"Saronite Vapors",0x20,12800,0
--4/29 22:38:40.135  SPELL_DAMAGE,0x0000000000000000,nil,0x80000000,0x0500000002BFA226,"Cimsir",0x514,63338,"Saronite Vapors",0x20,20480,8387,32,5120,0,0,nil,nil,nil
--	Saronite Vapors  -- General Vezax
		if (spellId==63337) then
			if addon.db.profile.saronite_vapor then
				if (amount == 12800) then
					addon:AddFail(destName,L["moving"],spellId)
				end
			end
		end
	end
end

if (type=="SPELL_DISPEL") then
	local extraSpellId = select(12,...)
--4/16 16:22:38.229  SPELL_DISPEL,0x05000000027EF48C,"Veev",0x511,0xF13000824A00E5AD,"Corrupted Servitor",0x8010a48,47488,"Shield Slam",0x1,63559,"",8,BUFF
-- Bind Life -- Freya Trash		
	if (extraspellId==63559) then
		if addon.db.profile.bind_life then
			addon:AddFail(sourceName,L["not_dispelling"],extraspellId)
		end
	end
end

if (type=="SPELL_INTERRUPT") then
	local spellId = select(9,...)
	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
--3/16 19:50:05.275  SPELL_INTERRUPT,0x0000000000000000,nil,0x80000000,0x01000000007CED8E,"Mute",0x514,62681,"Flame Jets",0x4,49238,"Lightning Bolt",8
-- Flame Jets -- Ignis
		if (spellId==62681) then					
			if addon.db.profile.flame_jets then
				addon:AddFail(destName,L["not_casting"],spellId)
			end
-- Ground Tremor -- Freya and Elder							
		elseif ((spellId==62859) or (spellId==62437) or (spellId==62325) or (spellId==62932)) then
			if addon.db.profile.ground_tremor then
				addon:AddFail(destName,L["not_casting"],spellId)
			end
		end
	end
end

if (type=="SPELL_AURA_APPLIED") then
	local spellId = select(9,...)
---- XT-002 ----
--6/8 20:04:58.937  SPELL_AURA_APPLIED,0xF15000820D00231E,"XT-002 Deconstructor",0x10a48,0x0300000001720FE7,"Methoz",0x512,63018,"Searing Light",0x42,DEBUFF
--6/10 19:52:15.015  SPELL_AURA_APPLIED,0xF15000820D017A0B,"XT-002 Deconstructor",0x10a48,0x0300000001D2BD79,"Korta",0x514,65121,"Searing Light",0x42,DEBUFF
-- Searing Light - XT-002	
	if ((spellId==63018) or (spellId==65121)) then
		if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
			searing_applied = timestamp		
		end		
---- THE IRON COUNCIL ----
--6/25 21:26:13.890  SPELL_AURA_APPLIED,0xF130008063022707,"Steelbreaker",0x10a48,0x0300000001D63C15,"Clozern",0x512,64637,"Overwhelming Power",0x1,DEBUFF		
-- Overwhelming Power -- The Iron Council
	elseif (spellId==64637) or (spellId==61888) then
		if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
			if UnitHealthMax(destName) <= 35000 then
				addon:AddFail(destName,GetSpellInfo(spellId),spellId)
			end
		end
-- MIMIRON --
--6/4 22:35:49.687  SPELL_AURA_APPLIED,0xF13000856307C936,"Emergency Fire Bot",0xa48,0x03000000002F5BD5,"Muqq",0x512,64616,"Deafening Siren",0x1,DEBUFF
-- Deafening Siren -- Mimiron
	elseif spellId==64616 then
		if addon.db.profile.deafening_siren then
			addon:AddFail(destName,L["moving"],spellId)
		end		
---- HODIR ----
--2/26 20:15:03.498  SPELL_AURA_APPLIED,0x0000000000000000,nil,0x80000000,0x01000000007C66FF,"Museedad",0x4000514,61969,"Flash Freeze",0x10,DEBUFF
-- Flash Freez -- Hodir
--add a check for hodir's room name		
	elseif ((spellId==61969) or (spellId==61990)) then
		if addon.db.profile.flash_freez then
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				if (GetMinimapZoneText()==L["The Halls of Winter"]) then -- makes sure not on yogg
					addon:AddFail(destName,L["moving"],spellId)
				end
			end		
		end
---- FREYA ----
--6/4 20:54:30.328  SPELL_AURA_APPLIED,0xF1300081B3070438,"Ancient Conservator",0x10a48,0x03000000022B3CAF,"Implied",0x512,63571,"Nature's Fury",0x8,DEBUFF		
--6/8 20:57:46.750  SPELL_AURA_APPLIED,0xF1300081B3007F9F,"Ancient Conservator",0x10a48,0x0300000002289DD9,"Spritzor",0x512,62589,"Nature's Fury",0x8,DEBUFF
	elseif ((spellId==63571) or (spellId==62589)) then
		if addon.db.profile.natures_fury then
			if fury_on == nil then
				fury_on = destName
				ChargeCounter[fury_on] = timestamp
			elseif fury_on1 == nil then
				fury_on1 = destName
				ChargeCounter[fury_on1] = timestamp
			elseif fury_on2 == nil then
				fury_on2 = destName
				ChargeCounter[fury_on2] = timestamp
			end		
		end
---- MIMIRON ----
--6/4 23:12:36.921  SPELL_AURA_APPLIED,0xF130008517081A94,"Leviathan Mk II",0xa48,0x0300000002171BAB,"Fenita",0x514,65026,"Napalm Shell",0x4,DEBUFF
--6/8 21:19:22.781  SPELL_AURA_APPLIED,0xF130008517000764,"Leviathan Mk II",0xa48,0x0300000001E5B381,"Baotn",0x2000514,63666,"Napalm Shell",0x4,DEBUFF
-- Napalm Shell -- Mimiron
	elseif ((spellId==65026) or (spellId==63666)) then
		if addon.db.profile.napalm_shell then
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				if LastEvent3 == nil then
					LastEvent3 = timestamp
				else
					if (timestamp - LastEvent3) < 1 then
						if not shock_blast_casting then
							addon:AddFail(destName,L["moving"],spellId)
						end
					end
					LastEvent3 = timestamp
				end		
			end	
		end
---- YOGG-SARON ----			
--5/11 19:02:48.828  SPELL_AURA_APPLIED,0x0300000001D2A4D6,"Mackzter",0x8000514,0x0300000001D32634,"Aryu",0x514,63881,"Malady of the Mind",0x20,DEBUFF
-- Malady of the Mind -- Yogg-Saron		
	elseif (spellId==63881) then
		if addon.db.profile.malady_of_the_mind then
			if bit.band(sourceFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				addon:AddFail(destName,L["moving"],spellId)
			end
		end
--5/25 18:04:24.234  SPELL_AURA_APPLIED,0x0000000000000000,nil,0x80000000,0x0300000001720FE7,"Methoz",0x514,63120,"Insane",0x20,DEBUFF
-- Insane -- Yogg-Saron -- when you loose all sanity		
	elseif (spellId==63120) then
		if addon.db.profile.insane then
			addon:AddFail(destName,L["sanity"],spellId)
		end
---- ALGALON ----
--5/29 23:06:31.750  SPELL_AURA_APPLIED,0x03000000000B526B,"Kungen",0x514,0x03000000000B526B,"Kungen",0x514,62169,"Black Hole",0x40,DEBUFF
-- Black Hole -- Algalon
	elseif (spellId==62169) then
		if addon.db.profile.black_hole then
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				if not bigbang_is_casting then
					addon:AddFail(destName,L["moving"],spellId)
				end
			end
		end
	end
end

---- ALGALON ----
--5/29 23:06:28.062  SPELL_CAST_START,0xF13000806703F29B,"Algalon the Observer",0x10a28,0x0000000000000000,nil,0x80000000,64584,"Big Bang",0x1
-- Big Bang -- Algalon -- for the Black Hole fail
if (type=="SPELL_CAST_START") then
	local spellId = select(9,...)
	if ((spellId == 64584) or (spellId == 64443)) then
		if addon.db.profile.big_bang then
			bigbang_is_casting = true
			addon:ScheduleTimer("NotBigBang", self.NotBigBang, 8)
		end
---- MIMIRON ----
--6/10 22:24:06.562  SPELL_CAST_START,0xF15000829803A324,"Leviathan Mk II",0x10a48,0x0000000000000000,nil,0x80000000,63631,"Shock Blast",0x8	
-- Shock Blast -- Mimiron
	elseif (spellId==63631) then
		if addon.db.profile.shock_blast then
			shock_blast_casting = true
			addon:ScheduleTimer("ShockBlastCasting", self.ShockBlastCasting, 6)
		end
---- AURIAYA ----		
--6/10 21:10:42.671  SPELL_CAST_START,0xF1300082EB003A16,"Auriaya",0x10a48,0x0000000000000000,nil,0x80000000,64688,"Sonic Screech",0x1
-- Sonic Screech -- Auriaya
	elseif ((spellId==64688) or (spellId==64422)) then
		if addon.db.profile.sonic_screech then
			not_failed_on_sonic_counter = 1
			addon:ScheduleTimer("FailedOnSonic", self.FailedOnSonic, 4)
		end
	end
end
		

---- GENERAL VEZAX ----	
if (type=="SPELL_HEAL") then
	local spellId,_,_,amount = select(9,...)
-- General healing himself	
	if spellId == 63278 then
		if addon.db.profile.general_healing_himself then
			heal_amount = heal_amount + amount
		end
	end
end

if (type=="SPELL_AURA_REMOVED") then
	local spellId = select(9,...)
-- Aura removed from the player
	if spellId == 63276 then
		if addon.db.profile.general_healing_himself then
			if heal_amount > addon.db.profile.general_healing_himself_value then			
				addon:AddFail(destName,L["moving"].." ("..heal_amount..") ".."("..(GetSpellInfo(spellId))..")",spellId)
			end
			heal_amount = 0
		end
---- XT-002 ----
--6/10 19:52:24.140  SPELL_AURA_REMOVED,0xF15000820D017A0B,"XT-002 Deconstructor",0x10a48,0x0300000001D2BD79,"Korta",0x8000514,65121,"Searing Light",0x42,DEBUFF
--6/8 20:05:07.890  SPELL_AURA_REMOVED,0xF15000820D00231E,"XT-002 Deconstructor",0x10a48,0x0300000001720FE7,"Methoz",0x512,63018,"Searing Light",0x42,DEBUFF
-- Searing Light - XT-002	
	elseif ((spellId==65121) or (spellId==63018)) then
		if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
			searing_applied = nil		
		end				
	elseif ((spellId==63571) or (spellId==62589)) then
		if addon.db.profile.natures_fury then
			if destName == fury_on then
				fury_on = nil
			elseif destName == fury_on1 then
				fury_on1 = nil
			elseif destName == fury_on2 then
				fury_on2 = nil
			end
		end
-- Hardened Bark -- Freya -- Snaplasher		
	elseif (spellId==64190) or (spellId==62663) then
		hardened_bark = false
-- Focused Anger -- Yogg-Saron -- Crusher Tentacle		
	elseif (spellId==57689) then
		focused_anger = false
	end
end

-------------------------------------
------ VAULT OF THE ANCIENTS---------
-------------------------------------

if (type=="SPELL_DAMAGE") then
	local spellId,_,_,amount = select(9,...)
	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
-- EMALON THE STORM WATCHER
--6/3 17:15:38.671  SPELL_DAMAGE,0xF1300084C9000A8A,"Emalon the Storm Watcher",0x10a48,0x0300000001D32634,"Aryu",0x514,65279,"Lightning Nova",0x8,7673,0,8,1920,0,0,nil,nil,nil	
-- Lightning Nova -- Emalon the Storm Watcher
		if ((spellId==65279) or (spellId==64216)) then	
			if addon.db.profile.lightning_nova then
				if amount >= addon.db.profile.lightning_nova_value then
					addon:AddFail(destName,L["moving"],spellId)
				end
			end
		end
	end
end

-------------------------------------
------		NAXXRAMAS		---------
-------------------------------------
--6/1 21:40:48.953  SPELL_DISPEL,0x03000000017F6EE9,"Calebx",0x511,0x0300000001D573D3,"Myumz",0x514,51886,"Cleanse Spirit",0x8,28169,"Mutating Injection",1,DEBUFF
-- Mutating Injection -- Grobbolus --
if (type=="SPELL_DISPEL") then
	local extraspellId = select(12,...)	
	if (extraspellId==28169) then
		if addon.db.profile.mutating_injection then
			addon:AddFail(sourceName,L["not_dispelling"],extraspellId)
		end
	end
end

if (type=="SPELL_DAMAGE") then
	local spellId,_,_,_,overkill = select(9,...)
	if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
-- Flame Tsunami -- Sartharion	
		if (spellId==57491) then
			if addon.db.profile.flame_tsunami then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					addon:AddFail(destName,L["moving"],spellId)
				else
					if (timestamp - LastEvent1[destName]) > 10 then
						addon:AddFail(destName,L["moving"],spellId)
					end
					LastEvent1[destName] = timestamp
				end	
			end
-- Eruption -- Heigan		
		elseif (spellId==29371) then
			if addon.db.profile.eruption then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					addon:AddFail(destName,L["moving"],spellId)
				else
					if (timestamp - LastEvent1[destName]) > 2 then
						addon:AddFail(destName,L["moving"],spellId)
					end
					LastEvent1[destName] = timestamp
				end
			end
--12/30 19:46:34.741  SPELL_DAMAGE,0xF130003E9B0084A5,"Living Poison",0xa48,0x0000000001D2D545,"Meka",0x514,28433,"Explode",0x8,28376,11654,8,3153,0,0,nil,nil,nil
-- Frogger		
		elseif (spellId==28433) then
			if addon.db.profile.frogger then
				addon:AddFail(destName,L["moving"],spellId)
			end
-- Mark -- Four Horsemen 			
		elseif (spellId==28836) then
			if addon.db.profile.mark then
				if overkill > 0 then
					addon:AddFail(destName,L["moving"],spellId)
				end
			end
-- Void Zones	
		elseif (spellId==28865) then
			if addon.db.profile.hm_void then
				if LastEvent1[destName] == nil then
					LastEvent1[destName] = timestamp
					addon:AddFail(destName,L["moving"],spellId)
				else
					if (timestamp - LastEvent1[destName]) > 3 then
						addon:AddFail(destName,L["moving"],spellId)
					end
					LastEvent1[destName] = timestamp
				end
			end
		elseif (spellId==27812) then
			if addon.db.profile.kt_void then
				addon:AddFail(destName,L["moving"],spellId)
			end
		elseif ((spellId==59128) or (spellId==57581)) then
			if addon.db.profile.sartharion_void then
				addon:AddFail(destName,L["moving"],spellId)
			end
-- Frost Breath -- Sapphiron		
		elseif (spellId==28524) then
			if addon.db.profile.frost_breath then
				addon:AddFail(destName,L["moving"],spellId)
			end
		end	
	end
end

if (type=="SPELL_CAST_START") then
	local spellId = select(9,...)
	if (spellId==28089) then
		ChargeCounter = {}
		ThaddiusAlive = true;
	end
end

if (type=="UNIT_DIED") then
	if (destName==BB["Thaddius"]) then
		ThaddiusAlive = false;
	end
end

if (destName==BB["Stalagg"]) or (destName==BB["Feugen"]) then
	WightLastAction = timestamp
end

if (ThaddiusAlive) then
	if (type=="SPELL_DAMAGE") then
		local spellId = select(9,...)
		if bit.band(sourceFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
			if ((spellId==28062) or (spellId==28085)) then
				if addon.db.profile.polarity_shift then
					if ChargeCounter[sourceName] == nil then
						ChargeCounter[sourceName] = 1
						LastEvent1[sourceName] = timestamp
					elseif (timestamp - LastEvent1[sourceName]) < 2 then
						ChargeCounter[sourceName] = ChargeCounter[sourceName] + 1
						LastEvent1[sourceName] = timestamp
					else
						ChargeCounter[sourceName] = 1
						LastEvent1[sourceName] = timestamp
					end
					if ChargeCounter[sourceName] == 3 then
	-- Polarity Shift -- Thaddius			
						addon:AddFail(sourceName,L["left and right"],28089)
					end
				end
			end
		end
	end
end

if (timestamp - WightLastAction) < 60 then
	if (type=="SPELL_AURA_APPLIED") then
		local spellId = select(9,...)
		if (spellId==28801) then	
			if addon.db.profile.jumping then
				if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
					WightLastAction = timestamp
					if LastEvent2[destName] == nil then
	-- Jumping -- Thaddius				
						addon:AddFail(destName,L["jumping"],47018)
					elseif (timestamp - LastEvent2[destName]) > 5 then
						addon:AddFail(destName,L["jumping"],47018)
					end
				end
			end
		end
	elseif (type=="SPELL_AURA_REMOVED") then
		local spellId = select(9,...)
		if (spellId==28801) then
			if addon.db.profile.jumping then
				if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
					LastEvent2[destName] = timestamp
				end
			end
		end
	end
end
end

function addon:OnUNIT_AURA(event,unit)
-- Biting Cold -- Hodir -- Taken from BigWigs	
	local ID,name = nil,nil 	
	if unit:sub(0,4)=="raid" then	
		if unit:sub(5,7) ~= "pet" then
			ID = unit:sub(5)
		end
		if ID ~= nil then
			name = GetRaidRosterInfo(ID)
		end
		local _, _, _, stack = UnitDebuff(unit, cold)
		if stack and stack ~= lastCold then
			if addon.db.profile.cold and stack > addon.db.profile.cold_value then
				local Time = GetTime()
				if lastColdTime == nil then
					lastColdTime = Time
					addon:AddFail(name,L["moving"],62039)
				elseif Time - lastColdTime > 4.000 then
					addon:AddFail(name,L["moving"],62039)
					lastColdTime = Time
				end					
			end
			lastCold = stack
		end	
	end
end

function addon:Wipe()
	addon:Msg(L["susped"],addon.db.profile.announce)
	addon:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	addon:UnregisterEvent("UNIT_AURA")
	addon:ScheduleTimer("ResumeFailReporting", self.Resume, 60)
end

function addon:FailedOnSonic()
   local num = GetNumRaidMembers()   
   local difficulty = GetCurrentDungeonDifficulty()
   local name = {}   
   for i = 1, num do      
      name[i]=select(1,GetRaidRosterInfo(i))
   end
   for i=1, #name do
      for j=1, #not_failed_on_sonic do
         if name[i] == not_failed_on_sonic[j]  then
            name[i] = nil
         end         
      end
      if name[i] ~= nil then  
		if UnitIsDeadOrGhost(name[i]) == nil then
			local _,_,group,_,_,_,zone,online=GetRaidRosterInfo(i)
			if difficulty == 2 then
				if group < 6 and zone == BZ["Ulduar"] and online then
					addon:AddFail(name[i],L["moving"],64688)
				end
			elseif difficulty == 1 then
				if group < 3 and zone == BZ["Ulduar"] and online then
					addon:AddFail(name[i],L["moving"],64688)
				end
			end
		end	  
      end
   end
    not_faild_on_sonic = {}
end

function addon:NotBigBang()
	bigbang_is_casting = false
end

function addon:ChainLightningCD()
	three_people_hugging = false
end

function addon:Resume()
	addon:Msg(L["resume"],addon.db.profile.announce)
	addon:RegisterEvent("UNIT_AURA")
	addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLog")	
end

function addon:CheckForWipe()
	local wipe = true
	local num = GetNumRaidMembers()
	for i = 1, num do
		local name = GetRaidRosterInfo(i)		
		if UnitAffectingCombat(name) ~= nil then
			wipe = false
		end
	end
	return wipe
end

function addon:ShockBlastCasting()
	shock_blast_casting = false
end

function addon:GravityBomb()
	FailedatGravityBomb = false
end

function addon:KeepCheckingForWipe()
	local wipe = false
	if not addon:CheckForWipe() then
		addon:ScheduleTimer("KeepCheckingForWipe", addon.KeepCheckingForWipe, 2)
	else
		wipe = true
		addon:ResetLunaticGaze()
	end
	return wipe
end

function addon:ResetLunaticGaze()
	LunaticGaze = {}
end

function addon:AnnounceAfter()
	if addon:CheckForWipe() then		
		for k, v in pairs(FailsForTheSession) do
		   event_fails1 = 0
		   failed_at1 = ""
		   for j, e in pairs(v) do
			  event_fails1 = e + event_fails1
			  failed_at1 = failed_at1..e.."x ("..(GetSpellInfo(j)).."), "
		   end
		   event_fails[k]= event_fails1
		   failed_at[k] = failed_at1      
		end
		Count = event_fails
		for k,v in pairs(Count) do
		   table.insert(sorttable, {k,v})
		end
		table.sort(sorttable,function(a,b) return a[2] > b[2] end)
		for k, v in pairs(sorttable) do
		   for j, e in pairs(failed_at) do
			  if v[1] == j then  
				if addon.db.profile.announce == "SELF" then
					DEFAULT_CHAT_FRAME:AddMessage(v[1].. L["failed"] ..v[2].."x ("..strsub(e,0,strlen(e)-3).."))")
				else
					SendChatMessage(v[1].. L["failed"] ..v[2].."x ("..strsub(e,0,strlen(e)-3).."))",addon.db.profile.announce,nil,addon.db.profile.channelnumber)
				end
			  end			  
		   end
		end
		FailsForTheSession = {} 
		event_fails ={} 
		sorttable ={} 
		failed_at={}
	else
		addon:ScheduleTimer("CheckForWipe", addon.AnnounceAfter, 2)
	end
end

function addon:AddFail(p,s,spellId)
	addon:Initialize()	
	if addon.db.profile.announce_style == "after" then
		if FailsForTheSession[p] == nil then			
			FailsForTheSession[p] = {}
		end
		if FailsForTheSession[p][spellId] == nil then
			FailsForTheSession[p][spellId] = 1		
		else
			FailsForTheSession[p][spellId] = FailsForTheSession[p][spellId] + 1			
		end
		if OFailCount[p] == nil then
			OFailCount[p] = 1
			FailCount[p] = 1
		else
			if FailCount[p] == nil then
				FailCount[p] = 1
			else
				OFailCount[p] = OFailCount[p] + 1
				FailCount[p] = FailCount[p] + 1
			end
		end
	elseif ((addon.db.profile.announce_style == "during") or (addon.db.profile.announce_style == "during_and_after")) then		
		if FailsForTheSession[p] == nil then			
			FailsForTheSession[p] = {}
		end
		if FailsForTheSession[p][spellId] == nil then
			FailsForTheSession[p][spellId] = 1		
		else
			FailsForTheSession[p][spellId] = FailsForTheSession[p][spellId] + 1			
		end
		if OFailCount[p] == nil then
			OFailCount[p] = 1
			FailCount[p] = 1	
		else
			if FailCount[p] == nil then
				FailCount[p] = 1
			else
				OFailCount[p] = OFailCount[p] + 1
				FailCount[p] = FailCount[p] + 1
			end
		end		
		if ((s==L["shaman_healing"]) or (s==L["moving"])) then			
			addon:SendChatMessage(p,s.." ("..(GetSpellInfo(spellId))..")")
		else			
			addon:SendChatMessage(p,s)
		end
	end
end

function addon:RemoveFail(p,t,w)
	if OFailCount[p] ~= nil then
		OFailCount[p] = OFailCount[p] - 1
		if FailCount[p] ~= nil then
			FailCount[p] = FailCount[p] - 1
		end
	end
	if addon.db.profile.announce == "SELF" then
		DEFAULT_CHAT_FRAME:AddMessage(L["removed"]..p)
	else
		SendChatMessage(L["removed"]..p,t,nil,w)
	end
end	

function addon:SendChatMessage(p,s)
	if not addon.db.profile.disabled then
		addon:Msg(p..L["fails_at"]..s.." ("..FailCount[p]..")",self.db.profile.announce)
	end
end

function addon:Msg(m,t)
	local w = self.db.profile.channelnumber
	if addon.db.profile.announce == "SELF" then
		DEFAULT_CHAT_FRAME:AddMessage(m)
	else
		SendChatMessage(m,t,nil,w)
	end
end

function addon:reset()
	FailCount = {}
	DEFAULT_CHAT_FRAME:AddMessage(L["reseted"]);
end

function addon:oreset()
	FailCount = {}
	OFailCount = {}
	DEFAULT_CHAT_FRAME:AddMessage(L["oreseted"]);
end
